@@ -8,6 +8,7 @@ import logging
 import os
 import xlrd,xlwt
 
+from functools import partial
 from glob import glob
 from shutil import rmtree
 from tempfile import mkdtemp
@@ -67,11 +68,13 @@ class BaseReader:
                     workbook.unload_sheet(sheet_x)
         filter.finish()
     
-class BaseFilter:
+class BaseFilterInterface:
     """
-    This is a simple filter that just calls the next filter in the
-    chain. The 'next' attribute is set up by the process method.
-    It can make a good base class for a new filter.
+    This is the filter interface that shows the correct way to call the 
+    next filter in the chain. 
+    The 'next' attribute is set up by the process method.
+    It can make a good base class for a new filter, but subclassing
+    BaseFilter below is often a better idea!
     """
 
     def start(self):
@@ -118,7 +121,6 @@ class BaseFilter:
         wtsheet_name - the name of the sheet into which content
                        should be written.
         """
-        self.rdsheet = rdsheet
         self.next.sheet(rdsheet,wtsheet_name)
        
     def set_rdsheet(self,rdsheet):
@@ -131,7 +133,6 @@ class BaseFilter:
                   this point forward should be read from.
 
         """
-        self.rdsheet = rdsheet
         self.next.set_rdsheet(rdsheet)
        
     def row(self,rdrowx,wtrowx):
@@ -183,6 +184,33 @@ class BaseFilter:
         """
         self.next.finish()
 
+class BaseFilter:
+
+    all_methods = (
+        'start',
+        'workbook',
+        'sheet',
+        'set_rdsheet',
+        'row',
+        'cell',
+        'finish',
+        )
+
+    def sheet(self,rdsheet,wtsheet_name):
+        self.rdsheet = rdsheet
+        self.next.sheet(rdsheet,wtsheet_name)
+       
+    def set_rdsheet(self,rdsheet):
+        self.rdsheet = rdsheet
+        self.next.set_rdsheet(rdsheet)
+
+    def __getattr__(self,name):
+        if name not in self.all_methods:
+            raise AttributeError(name)
+        actual = getattr(self.next,name)
+        setattr(self,name,actual)
+        return actual
+
 class BaseWriter:
     """
     This is the base writer that copies all data and formatting from
@@ -226,6 +254,10 @@ class BaseWriter:
             if self.close_after_write:
                 stream.close()
             del self.wtbook
+            del self.rdbook
+            del self.rdsheet
+            del self.wtsheet
+            del self.style_list
 
     def workbook(self,rdbook,wtbook_name):
         self.close()        
@@ -532,33 +564,13 @@ class XLWTWriter(BaseWriter):
             self.output.append((self.wtname,self.wtbook))
             del self.wtbook
 
-class MethodFilterMethod:
-
-    def __init__(self,mf,name):
-        self.mf,self.name = mf,name
-
-    def __call__(self,*args):
-        if self.name in self.mf.call_on:
-            self.mf.method(self.name,*args)
-        getattr(self.mf.next,self.name)(*args)
-        
-class MethodFilter:
+class MethodFilter(BaseFilter):
     """
     This is a base class that implements functionality for filters
     that want to do a common task such as logging, printing or memory
     usage recording on certain calls configured at filter instantiation
     time.
     """
-
-    all_methods = (
-        'start',
-        'workbook',
-        'sheet',
-        'set_rdsheet',
-        'row',
-        'cell',
-        'finish',
-        )
     
     def method(self,name,*args):
         """
@@ -575,10 +587,11 @@ class MethodFilter:
         for name in methods:
             if name not in self.all_methods:
                 raise ValueError('%r is not a valid method name'%(name,))
-        self.call_on = methods
+            setattr(self,name,partial(self.caller,name))
 
-    def __getattr__(self,name):
-        return MethodFilterMethod(self,name)
+    def caller(self,name,*args):
+        self.method(name,*args)
+        getattr(self.next,name)(*args)
 
 class Echo(MethodFilter):
 
@@ -593,9 +606,9 @@ class Echo(MethodFilter):
         
 try:
     from guppy import hpy
-    h = hpy()
+    guppy = True
 except ImportError:
-    h = None
+    guppy = False
     
 class MemoryLogger(MethodFilter):
 
@@ -604,9 +617,15 @@ class MemoryLogger(MethodFilter):
         self.path = path
         
     def method(self,name,*args):
-        if h is not None:
-            h.heap().stat.dump(self.path)
-        
+        if guppy:
+            # We instantiate the heapy environment here
+            # so that the memory it consumes doesn't hang
+            # around for the whole process
+            hpy().heap().stat.dump(self.path)
+        else:
+            logger.error('guppy is not availabe, cannot log memory usage!')
+            
+            
 class ErrorFilter(BaseReader,BaseWriter):
 
     temp_path = None
@@ -682,6 +701,9 @@ class ErrorFilter(BaseReader,BaseWriter):
         else:
             self(self.next)
         self.start(create=False)
+        for attr in ('rdbook','rdsheet'):
+            if hasattr(self,attr):
+                delattr(self,attr)
 
 class Range(object):
     __slots__ = ('rsn','rr','rc','wr','wc','r','c')
@@ -781,6 +803,7 @@ class ColumnTrimmer(BaseFilter):
                                                           
     def finish(self):
         self.flush()
+        del self.rdbook
         self.next.finish()
         
 def process(reader,*chain):
